require "import"
import "android.widget.*"
import "android.view.*"

-- 方向常量
local DIRECTION_NONE = 0
local DIRECTION_LEFT = 1
local DIRECTION_RIGHT = 2
local DIRECTION_UP = 3
local DIRECTION_DOWN = 4

-- 触摸参数
local startX, startY
local minDistance = 50  -- 滑动最小距离
local maxClickDistance = 20  -- 点击最大允许移动距离(超过则认为是滑动)
local lastDirection = DIRECTION_NONE

-- 双击检测参数
local lastSingleTapTime = 0
local lastDoubleTapTime = 0
local fingerCount = 0
local DOUBLE_TAP_TIMEOUT = 200  -- 双击间隔(毫秒)
local SINGLE_TAP_TIMEOUT = 300  -- 单击超时时间(毫秒)
local isClick = false  -- 标记是否为点击操作
local isSliding = false  -- 标记是否为滑动操作

layout = {
LinearLayout,
{
TextView,
id = "btnSwipe",
layout_width = "100%w",
layout_height = "100%h",
text = "媒体控制面板",
gravity = "center",
background="#8800ff00",
},
}

mtmb = LuaDialog()
mtmb.setView(loadlayout(layout))
mtmb.setOnDismissListener(function()
service.setTouchExplorationMode(true)
service.asyncSpeak("退出")
end)
mtmb.show()
service.setTouchExplorationMode(false)

-- 判断方向
local function getDirection(dx, dy)
if math.abs(dx) > math.abs(dy) then
if dx > minDistance then return DIRECTION_RIGHT end
if dx < -minDistance then return DIRECTION_LEFT end
else
if dy > minDistance then return DIRECTION_DOWN end
if dy < -minDistance then return DIRECTION_UP end
end
return DIRECTION_NONE
end

btnSwipe.onTouch = function(v, event)
local action = event.getActionMasked()
local currentTime = os.clock() * 1000  -- 当前时间(毫秒)
local x, y = event.getX(), event.getY()

-- 检测手指数量
if action == MotionEvent.ACTION_DOWN or action == MotionEvent.ACTION_POINTER_DOWN then
fingerCount = event.getPointerCount()
startX = x
startY = y
isClick = true
isSliding = false
lastDirection = DIRECTION_NONE
end

-- 处理移动事件
if action == MotionEvent.ACTION_MOVE then
local dx = x - startX
local dy = y - startY

-- 如果移动距离超过阈值，则认为是滑动
if math.abs(dx) > maxClickDistance or math.abs(dy) > maxClickDistance then
isClick = false
isSliding = true
end

-- 滑动处理
if isSliding then
local direction = getDirection(dx, dy)
if direction ~= DIRECTION_NONE and direction ~= lastDirection then
-- 单指滑动
if fingerCount == 1 then
if direction == DIRECTION_LEFT then
service.execute("快退", node)
elseif direction == DIRECTION_RIGHT then
service.execute("快进", node)
elseif direction == DIRECTION_UP then
service.execute("上一曲", node)
elseif direction == DIRECTION_DOWN then
service.execute("下一曲", node)
end
-- 双指滑动
elseif fingerCount == 2 then
-- 上下滑动控制音量
if direction == DIRECTION_UP then
service.execute("增大音量", node)
elseif direction == DIRECTION_DOWN then
service.execute("减小音量", node)
-- 左右滑动音频闪避开关
elseif direction == DIRECTION_LEFT then
service.setAudioFocus(false)
service.asyncSpeak("音频闪避关闭")
elseif direction == DIRECTION_RIGHT then
service.setAudioFocus(true)
service.asyncSpeak("音频闪避打开")
end
end
lastDirection = direction
startX = x
startY = y
end
end
return true
end

-- 处理抬起事件
if action == MotionEvent.ACTION_UP or action == MotionEvent.ACTION_POINTER_UP then
-- 如果是点击操作(不是滑动)且移动距离在点击范围内
if isClick and (math.abs(x - startX) <= maxClickDistance and math.abs(y - startY) <= maxClickDistance) then
-- 单指点击
if fingerCount == 1 then
if currentTime - lastSingleTapTime < DOUBLE_TAP_TIMEOUT then
-- 单指双击
service.execute("播放暂停", node)
lastSingleTapTime = 0
else
-- 单指单击，记录时间
lastSingleTapTime = currentTime
end
-- 双指点击
elseif fingerCount == 2 then
if currentTime - lastDoubleTapTime < DOUBLE_TAP_TIMEOUT then
-- 双指双击
service.execute("返回", node)
lastDoubleTapTime = 0
else
-- 双指单击，记录时间
lastDoubleTapTime = currentTime
end
end
end

-- 更新手指计数
fingerCount = event.getPointerCount() - (action == MotionEvent.ACTION_UP and 1 or 0)
return true
end

return true
end
return true